@@ -1,40 +1,45 @@
-""" Internal utilities for helping to manage xarray and numpy
-objects used throughout GCPy """
-
+"""
+Internal utilities for helping to manage xarray and numpy
+objects used throughout GCPy
+"""
 
 import os
 import warnings
-import yaml
 import shutil
+import yaml
 import numpy as np
 import xarray as xr
-import gcpy.constants as gcon
 from PyPDF2 import PdfFileWriter, PdfFileReader
 
-def convert_lon(data, dim='lon', format='atlantic', neg_dateline=True):
+def convert_lon(
+        data,
+        dim='lon',
+        format='atlantic',
+        neg_dateline=True
+):
     """
     Convert longitudes from -180..180 to 0..360, or vice-versa.
 
     Args:
         data: DataArray or Dataset
-             The container holding the data to be converted; the dimension indicated
-             by 'dim' must be associated with this container
+             The container holding the data to be converted; the dimension
+             indicated by 'dim' must be associated with this container
 
     Keyword Args (optional):
         dim: str
              Name of dimension holding the longitude coordinates
              Default value: 'lon'
         format: str
-             Control whether or not to shift from -180..180 to 0..360 ('pacific') or
-             from 0..360 to -180..180 ('atlantic')
+             Control whether or not to shift from -180..180 to 0..360 (
+             ('pacific') or from 0..360 to -180..180 ('atlantic')
              Default value: 'atlantic'
         neg_dateline: logical
-             If True, then the international dateline is set to -180 instead of 180.
+             If True, then the international dateline is set to -180
+             instead of 180.
              Default value: True
 
     Returns:
         data, with dimension 'dim' altered according to conversion rule
-
     """
 
     data_copy = data.copy()
@@ -74,7 +79,10 @@ def convert_lon(data, dim='lon', format='atlantic', neg_dateline=True):
     return data_copy
 
 
-def get_emissions_varnames(commonvars, template=None):
+def get_emissions_varnames(
+        commonvars,
+        template=None
+):
     """
     Will return a list of emissions diagnostic variable names that
     contain a particular search string.
@@ -107,7 +115,9 @@ def get_emissions_varnames(commonvars, template=None):
     return varnames
 
 
-def create_display_name(diagnostic_name):
+def create_display_name(
+        diagnostic_name
+):
     """
     Converts a diagnostic name to a more easily digestible name
     that can be used as a plot title or in a table of totals.
@@ -148,7 +158,12 @@ def create_display_name(diagnostic_name):
     return display_name
 
 
-def print_totals(ref, refstr, dev, devstr, f, masks=None):
+def print_totals(
+        ref,
+        dev,
+        f,
+        masks=None
+):
     """
     Computes and prints Ref and Dev totals (as well as the difference
     Dev - Ref) for two xarray DataArray objects.
@@ -156,14 +171,8 @@ def print_totals(ref, refstr, dev, devstr, f, masks=None):
     Args:
         ref: xarray DataArray
             The first DataArray to be compared (aka "Reference")
-        refstr: str
-            A string that can be used to identify refdata.
-            (e.g. a model version number or other identifier).
         dev: xarray DataArray
             The second DataArray to be compared (aka "Development")
-        devstr: str
-            A string that can be used to identify devdata
-            (e.g. a model version number or other identifier).
         f: file
             File object denoting a text file where output will be directed.
 
@@ -218,7 +227,7 @@ def print_totals(ref, refstr, dev, devstr, f, masks=None):
 
     # Special handling for totals
     if "_TOTAL" in diagnostic_name.upper():
-        print("-" * 83, file=f)
+        print("-"*83, file=f)
 
     # ==================================================================
     # Sum the Ref array (or set to NaN if missing)
@@ -274,7 +283,9 @@ def print_totals(ref, refstr, dev, devstr, f, masks=None):
     )
 
 
-def get_species_categories(benchmark_type="FullChemBenchmark"):
+def get_species_categories(
+        benchmark_type="FullChemBenchmark"
+):
     """
     Returns the list of benchmark categories that each species
     belongs to.  This determines which PDF files will contain the
@@ -300,7 +311,9 @@ def get_species_categories(benchmark_type="FullChemBenchmark"):
     return spc_cat_dict[benchmark_type]
 
 
-def archive_species_categories(dst):
+def archive_species_categories(
+        dst
+):
     """
     Writes the list of benchmark categories to a YAML file
     named "benchmark_species.yml".
@@ -313,11 +326,18 @@ def archive_species_categories(dst):
     """
     spc_categories = "benchmark_categories.yml"
     src = os.path.join(os.path.dirname(__file__), spc_categories)
-    print("Archiving {} in {}".format(spc_categories, dst))
-    shutil.copyfile(src, os.path.join(dst, spc_categories))
+    copy = os.path.join(dst, spc_categories)
+    if not os.path.exists(copy):
+        print("\nArchiving {} in {}".format(spc_categories, dst))
+        shutil.copyfile(src, copy)
 
 
-def add_bookmarks_to_pdf(pdfname, varlist, remove_prefix="", verbose=False):
+def add_bookmarks_to_pdf(
+        pdfname,
+        varlist,
+        remove_prefix="",
+        verbose=False
+):
     """
     Adds bookmarks to an existing PDF file.
 
@@ -368,7 +388,11 @@ def add_bookmarks_to_pdf(pdfname, varlist, remove_prefix="", verbose=False):
 
 
 def add_nested_bookmarks_to_pdf(
-    pdfname, category, catdict, warninglist, remove_prefix=""
+        pdfname,
+        category,
+        catdict,
+        warninglist,
+        remove_prefix=""
 ):
     """
     Add nested bookmarks to PDF.
@@ -417,8 +441,7 @@ def add_nested_bookmarks_to_pdf(
             for varname in catdict[category][subcat]:
                 if varname in warninglist:
                     continue
-                else:
-                    numvars += 1
+                numvars += 1
         else:
             continue
         if numvars == 0:
@@ -458,7 +481,12 @@ def add_nested_bookmarks_to_pdf(
     pdfobj.close()
 
 
-def add_missing_variables(refdata, devdata, verbose=False, **kwargs):
+def add_missing_variables(
+        refdata,
+        devdata,
+        verbose=False,
+        **kwargs
+):
     """
     Compares two xarray Datasets, "Ref", and "Dev".  For each variable
     that is present  in "Ref" but not in "Dev", a DataArray of missing
@@ -548,7 +576,9 @@ def add_missing_variables(refdata, devdata, verbose=False, **kwargs):
     return refdata, devdata
 
 
-def reshape_MAPL_CS(da):
+def reshape_MAPL_CS(
+        da
+):
     """
     Reshapes data if contains dimensions indicate MAPL v1.0.0+ output
     Args:
@@ -562,7 +592,7 @@ def reshape_MAPL_CS(da):
 
     # Suppress annoying future warnings for now
     warnings.filterwarnings("ignore", category=FutureWarning)
-    
+
     if type(da) != np.ndarray:
         vdims = da.dims
         if "nf" in vdims and "Xdim" in vdims and "Ydim" in vdims:
@@ -580,7 +610,10 @@ def reshape_MAPL_CS(da):
     return da
 
 
-def get_diff_of_diffs(ref, dev):
+def get_diff_of_diffs(
+        ref,
+        dev
+):
     """
     Generate datasets containing differences between two datasets
 
@@ -645,7 +678,13 @@ def get_diff_of_diffs(ref, dev):
     return absdiffs, fracdiffs
 
 
-def slice_by_lev_and_time(ds, varname, itime, ilev, flip):
+def slice_by_lev_and_time(
+        ds,
+        varname,
+        itime,
+        ilev,
+        flip
+):
     """
     Slice a DataArray by desired time and level.
 
@@ -671,21 +710,20 @@ def slice_by_lev_and_time(ds, varname, itime, ilev, flip):
         if flip:
             maxlev_i = len(ds['lev'])-1
             return ds[varname].isel(time=itime, lev=maxlev_i - ilev)
-        else:
-            return ds[varname].isel(time=itime, lev=ilev)
-    elif ("time" not in vdims or itime == -1) and "lev" in vdims:
+        return ds[varname].isel(time=itime, lev=ilev)
+    if ("time" not in vdims or itime == -1) and "lev" in vdims:
         if flip:
             maxlev_i = len(ds['lev'])-1
             return ds[varname].isel(lev=maxlev_i - ilev)
-        else:
-            return ds[varname].isel(lev=ilev)
-    elif "time" in vdims and "lev" not in vdims and itime != -1:
+        return ds[varname].isel(lev=ilev)
+    if "time" in vdims and "lev" not in vdims and itime != -1:
         return ds[varname].isel(time=itime)
-    else:
-        return ds[varname]
+    return ds[varname]
 
 
-def rename_and_flip_gchp_rst_vars(ds):
+def rename_and_flip_gchp_rst_vars(
+        ds
+):
     '''
     Transforms a GCHP restart dataset to match GCC names and level convention
 
@@ -716,7 +754,10 @@ def rename_and_flip_gchp_rst_vars(ds):
     return ds
 
 
-def dict_diff(dict0, dict1):
+def dict_diff(
+        dict0,
+        dict1
+):
     """
     Function to take the difference of two dict objects.
     Assumes that both objects have the same keys.
@@ -736,7 +777,12 @@ def dict_diff(dict0, dict1):
     return result
 
 
-def compare_varnames(refdata, devdata, refonly=[], devonly=[], quiet=False):
+def compare_varnames(
+        refdata,
+        devdata,
+        refonly=None,
+        devonly=None,
+        quiet=False):
     """
     Finds variables that are common to two xarray Dataset objects.
 
@@ -895,7 +941,10 @@ def compare_stats(refdata, refstr, devdata, devstr, varname):
     print("    {}:  {}".format(devstr, np.round(devvar.values.sum(), 20)))
 
 
-def convert_bpch_names_to_netcdf_names(ds, verbose=False):
+def convert_bpch_names_to_netcdf_names(
+        ds,
+        verbose=False
+):
     """
     Function to convert the non-standard bpch diagnostic names
     to names used in the GEOS-Chem netCDF diagnostic outputs.
@@ -924,7 +973,7 @@ def convert_bpch_names_to_netcdf_names(ds, verbose=False):
     # Now read from YAML file (bmy, 4/5/19)
     bpch_to_nc_names = "bpch_to_nc_names.yml"
     yamlfile = os.path.join(os.path.dirname(__file__), bpch_to_nc_names)
-    names = yaml.load(open(yamlfile))
+    names = yaml.load(open(yamlfile), Loader=yaml.FullLoader)
 
     # define some special variable to overwrite above
     special_vars = {
@@ -1011,18 +1060,18 @@ def convert_bpch_names_to_netcdf_names(ds, verbose=False):
 
             # These categories use append
             if oldid in [
-                "IJ_AVG_S_",
-                "RN_DECAY_",
-                "WETDCV_S_",
-                "WETDLS_S_",
-                "BXHGHT_S_",
-                "DAO_3D_S_",
-                "PL_SUL_",
-                "CV_FLX_S_",
-                "EW_FLX_S_",
-                "NS_FLX_S_",
-                "UP_FLX_S_",
-                "MC_FRC_S_",
+                    "IJ_AVG_S_",
+                    "RN_DECAY_",
+                    "WETDCV_S_",
+                    "WETDLS_S_",
+                    "BXHGHT_S_",
+                    "DAO_3D_S_",
+                    "PL_SUL_",
+                    "CV_FLX_S_",
+                    "EW_FLX_S_",
+                    "NS_FLX_S_",
+                    "UP_FLX_S_",
+                    "MC_FRC_S_",
             ]:
                 newvar = newid + "_" + varstr
 
@@ -1095,27 +1144,47 @@ def convert_bpch_names_to_netcdf_names(ds, verbose=False):
 
 
 def get_lumped_species_definitions():
+    """
+    Returns lumped species definitions from a YAML file.
+
+    Returns:
+        lumped_spc_dict : dict of str
+            Dictionary of lumped species
+    """
     lumped_spc = "lumped_species.yml"
     yamlfile = os.path.join(os.path.dirname(__file__), lumped_spc)
     with open(yamlfile, "r") as f:
-        lumped_spc_dict = yaml.load(f.read())
+        lumped_spc_dict = yaml.load(f.read(), Loader=yaml.FullLoader)
     return lumped_spc_dict
 
 
-def archive_lumped_species_definitions(dst):
+def archive_lumped_species_definitions(
+        dst
+):
+    """
+    Archives lumped species definitions to a YAML file.
+
+    Args:
+        dst : str
+            Name of the folder where the YAML file containing
+            benchmark categories ("benchmark_species.yml")
+            will be written.
+    """
     lumped_spc = "lumped_species.yml"
     src = os.path.join(os.path.dirname(__file__), lumped_spc)
-    print("Archiving {} in {}".format(lumped_spc, dst))
-    shutil.copyfile(src, os.path.join(dst, lumped_spc))
+    copy = os.path.join(dst, lumped_spc)
+    if not os.path.exists(copy):
+        print("\nArchiving {} in {}".format(lumped_spc, dst))
+        shutil.copyfile(src, copy)
 
 
 def add_lumped_species_to_dataset(
-    ds,
-    lspc_dict={},
-    lspc_yaml="",
-    verbose=True,
-    overwrite=False,
-    prefix="SpeciesConc_",
+        ds,
+        lspc_dict={},
+        lspc_yaml="",
+        verbose=False,
+        overwrite=False,
+        prefix="SpeciesConc_",
 ):
     """
     Function to calculate lumped species concentrations and add
@@ -1139,7 +1208,7 @@ def add_lumped_species_to_dataset(
             Default value: False
         verbose: bool
             Whether to print informational output.
-            Default value: True
+            Default value: False
         overwrite: bool
             Whether to overwrite an existing species dataarray in a dataset
             if it has the same name as a new lumped species. If False and
@@ -1168,66 +1237,80 @@ def add_lumped_species_to_dataset(
         lspc_dict = get_lumped_species_definitions()
     elif lspc_dict == {} and lspc_yaml != "":
         with open(lspc_yaml, "r") as f:
-            lspc_dict = yaml.load(f.read())
+            lspc_dict = yaml.load(f.read(), Loader=yaml.FullLoader)
 
-    # Get a dummy array to use for initialization
-    for var in ds.data_vars:
-        if prefix in var:
-            dummy_darr = ds[var]
-            break
+    # Make sure attributes are transferred when copying dataset / dataarrays
+    with xr.set_options(keep_attrs=True):
 
-    # Create a new dataset equivalent to the old
-    ds_new = ds
+        # Get a dummy array to use for initialization
+        dummy_darr = None
+        for var in ds.data_vars:
+            if prefix in var:
+                dummy_darr = ds[var]
+                break
+        if dummy_darr is None:
+            msg = "Invalid prefix: " + prefix
+            raise ValueError(msg)
 
-    for lspc in lspc_dict:
+        # Create a new dataset equivalent to the old
+        ds_new = ds.copy(deep=True)
 
-        # Assemble lumped species variable name
-        varname_new = prefix + lspc
+        # Free memory of original dataset
+        ds = xr.Dataset()
 
-        # Check if overlap with existing species
-        if varname_new in ds_new.data_vars and overwrite:
-            ds_new.drop(varname_new)
-        else:
-            assert(varname_new not in ds_new.data_vars), \
-                "{} already in dataset. To overwrite pass overwrite=True.".\
-                format(varname_new)
+        # Loop over lumped species
+        for lspc in lspc_dict:
 
-        # Verbose prints
-        if verbose:
-            print("Creating {}".format(varname_new))
+            # Assemble lumped species variable name
+            varname_new = prefix + lspc
 
-        # Initialize new dataarray
-        darr = dummy_darr
-        darr.name = varname_new
-        darr.values = np.full(darr.shape, 0.0)
+            # Check if overlap with existing species
+            if varname_new in ds_new.data_vars and overwrite:
+                ds_new.drop(varname_new)
+            else:
+                assert(varname_new not in ds_new.data_vars), \
+                    "{} already in dataset. To overwrite pass overwrite=True.".\
+                    format(varname_new)
 
-        # Loop over and sum constituent species values
-        num_spc = 0
-        for _, spc in enumerate(lspc_dict[lspc]):
-            varname = prefix + spc
-            if varname not in ds_new.data_vars:
-                print("Warning: {} needed for {} not in dataset.".
-                      format(spc, lspc))
-                continue
+            # Verbose prints
             if verbose:
-                print(" -> adding {} with scale {}".
-                      format(spc, lspc_dict[lspc][spc]))
-            darr.values = darr.values + \
-                ds_new[varname].values * lspc_dict[lspc][spc]
-            num_spc = num_spc + 1
+                print("Creating {}".format(varname_new))
 
-        # Replace values with NaN is no species found in dataset
-        if num_spc == 0:
-            print('No constituent species found in file. Setting to NaN.')
-            darr.values = np.full(darr.shape, np.nan)
+            # Initialize new dataarray for the lumped species
+            darr = dummy_darr
+            darr.name = varname_new
+            darr.values = np.full(darr.shape, 0.0)
 
-        # Merge new variable into dataset
-        ds_new = xr.merge([ds_new, darr])
+            # Loop over and sum constituent species values
+            num_spc = 0
+            for _, spc in enumerate(lspc_dict[lspc]):
+                varname = prefix + spc
+                if varname not in ds_new.data_vars:
+                    if verbose:
+                        print("Warning: {} needed for {} not in dataset.".
+                              format(spc, lspc))
+                    continue
+                if verbose:
+                    print(" -> adding {} with scale {}".
+                          format(spc, lspc_dict[lspc][spc]))
+                darr.values += ds_new[varname].values * lspc_dict[lspc][spc]
+                num_spc += 1
+
+            # Replace values with NaN is no species found in dataset
+            if num_spc == 0:
+                if verbose:
+                    print("No constituent species found! Setting to NaN.")
+                darr.values = np.full(darr.shape, np.nan)
+
+            # Merge new variable into dataset
+            ds_new = xr.merge([ds_new, darr])
 
     return ds_new
 
 
-def filter_names(names, text=""):
+def filter_names(
+        names,
+        text=""):
     """
     Returns elements in a list that match a given substring.
     Can be used in conjnction with compare_varnames to return a subset
@@ -1254,7 +1337,11 @@ def filter_names(names, text=""):
     return filtered_names
 
 
-def divide_dataset_by_dataarray(ds, dr, varlist=None):
+def divide_dataset_by_dataarray(
+        ds,
+        dr,
+        varlist=None
+):
     """
     Divides variables in an xarray Dataset object by a single DataArray
     object.  Will also make sure that the Dataset variable attributes
@@ -1310,7 +1397,11 @@ def divide_dataset_by_dataarray(ds, dr, varlist=None):
     return ds
 
 
-def get_shape_of_data(data, vertical_dim="lev", return_dims=False):
+def get_shape_of_data(
+        data,
+        vertical_dim="lev",
+        return_dims=False
+):
     """
     Convenience routine to return a the shape (and dimensions, if
     requested) of an xarray Dataset, or xarray DataArray.  Can also
@@ -1343,7 +1434,7 @@ def get_shape_of_data(data, vertical_dim="lev", return_dims=False):
     """
 
     # Validate the data argument
-    if isinstance(data, xr.Dataset) or isinstance(data, xr.DataArray):
+    if isinstance(data, (xr.Dataset, xr.DataArray)):
         sizelist = data.sizes
     elif isinstance(data, dict):
         sizelist = data
@@ -1368,11 +1459,12 @@ def get_shape_of_data(data, vertical_dim="lev", return_dims=False):
 
     if return_dims:
         return shape, dims
-    else:
-        return shape
+    return shape
 
 
-def get_area_from_dataset(ds):
+def get_area_from_dataset(
+        ds
+):
     """
     Convenience routine to return the area variable (which is
     usually called "AREA" for GEOS-Chem "Classic" or "Met_AREAM2"
@@ -1388,17 +1480,19 @@ def get_area_from_dataset(ds):
 
     if "Met_AREAM2" in ds.data_vars.keys():
         return ds["Met_AREAM2"]
-    elif "AREA" in ds.data_vars.keys():
+    if "AREA" in ds.data_vars.keys():
         return ds["AREA"]
-    else:
-        msg = (
-            'An area variable ("AREA" or "Met_AREAM2" is missing'
-            + " from this dataset!"
-        )
-        raise ValueError(msg)
+    msg = (
+        'An area variable ("AREA" or "Met_AREAM2" is missing'
+        + " from this dataset!"
+    )
+    raise ValueError(msg)
 
 
-def get_variables_from_dataset(ds, varlist):
+def get_variables_from_dataset(
+        ds,
+        varlist
+):
     """
     Convenience routine to return multiple selected DataArray
     variables from an xarray Dataset.  All variables must be
@@ -1431,7 +1525,13 @@ def get_variables_from_dataset(ds, varlist):
     return ds_subset
 
 
-def create_dataarray_of_nan(name, sizes, coords, attrs, vertical_dim="lev"):
+def create_dataarray_of_nan(
+        name,
+        sizes,
+        coords,
+        attrs,
+        vertical_dim="lev"
+):
     """
     Given an xarray DataArray dr, returns a DataArray object with
     the same dimensions, coordinates, attributes, and name, but
@@ -1494,7 +1594,11 @@ def create_dataarray_of_nan(name, sizes, coords, attrs, vertical_dim="lev"):
     )
 
 
-def check_for_area(ds, gcc_area_name="AREA", gchp_area_name="Met_AREAM2"):
+def check_for_area(
+        ds,
+        gcc_area_name="AREA",
+        gchp_area_name="Met_AREAM2"
+):
     """
     Makes sure that a dataset has a surface area variable contained
     within it.
@@ -1537,7 +1641,13 @@ def check_for_area(ds, gcc_area_name="AREA", gchp_area_name="Met_AREAM2"):
     return ds
 
 
-def get_filepath(datadir, col, date, is_gchp=False):
+def get_filepath(
+        datadir,
+        col,
+        date,
+        is_gchp=False,
+        gchp_format_is_legacy=False
+):
     """
     Routine to return file path for a given GEOS-Chem "Classic"
     (aka "GCC") or GCHP diagnostic collection and date.
@@ -1556,6 +1666,11 @@ def get_filepath(datadir, col, date, is_gchp=False):
             Set this switch to True to obtain file pathnames to
             GCHP diagnostic data files. If False, assumes GEOS-Chem "Classic"
 
+        gchp_format_is_legacy: bool
+            Set this switch to True to obtain GCHP file pathnames of
+            the legacy format for diagnostics, which do not match GC-Classic
+            filenames. Set to False to use same format as GC-Classic.
+
     Returns:
         path: str
             Pathname for the specified collection and date.
@@ -1564,6 +1679,7 @@ def get_filepath(datadir, col, date, is_gchp=False):
     # Set filename template, extension, separator, and date string from
     # the collection, date, and data directory arguments
     separator = "_"
+    extension = "z.nc4"
     date_str = np.datetime_as_string(date, unit="m")
     if is_gchp:
         if "Restart" in col:
@@ -1572,8 +1688,10 @@ def get_filepath(datadir, col, date, is_gchp=False):
             extension = ".nc4"
             date_str = np.datetime_as_string(date, unit="s")
         else:
-            file_tmpl = os.path.join(datadir, "GCHP.{}.".format(col))
-            extension = "z.nc4"
+            if gchp_format_is_legacy:
+                file_tmpl = os.path.join(datadir, "GCHP.{}.".format(col))
+            else:
+                file_tmpl = os.path.join(datadir, "GEOSChem.{}.".format(col))
     else:
         if "Emissions" in col:
             file_tmpl = os.path.join(datadir, "HEMCO_diagnostics.")
@@ -1581,7 +1699,8 @@ def get_filepath(datadir, col, date, is_gchp=False):
             separator = ""
         else:
             file_tmpl = os.path.join(datadir, "GEOSChem.{}.".format(col))
-            extension = "z.nc4"
+    if isinstance(date_str, np.str_):
+        date_str = str(date_str)
     date_str = date_str.replace("T", separator)
     date_str = date_str.replace("-", "")
     date_str = date_str.replace(":", "")
@@ -1591,7 +1710,13 @@ def get_filepath(datadir, col, date, is_gchp=False):
     return path
 
 
-def get_filepaths(datadir, collections, dates, is_gchp=False):
+def get_filepaths(
+        datadir,
+        collections,
+        dates,
+        is_gchp=False,
+        gchp_format_is_legacy=False
+):
     """
     Routine to return filepaths for a given GEOS-Chem "Classic"
     (aka "GCC") or GCHP diagnostic collection.
@@ -1610,6 +1735,11 @@ def get_filepaths(datadir, collections, dates, is_gchp=False):
             Set this switch to True to obtain file pathnames to
             GCHP diagnostic data files. If False, assumes GEOS-Chem "Classic"
 
+        gchp_format_is_legacy: bool
+            Set this switch to True to obtain GCHP file pathnames of
+            the legacy format for diagnostics, which do not match GC-Classic
+            filenames. Set to False to use same format as GC-Classic.
+
     Returns:
         paths: 2D list of str
             A list of pathnames for each specified collection and date.
@@ -1634,6 +1764,8 @@ def get_filepaths(datadir, collections, dates, is_gchp=False):
     # ==================================================================
     for c, collection in enumerate(collections):
 
+        separator = "_"
+        extension = "z.nc4"
         if is_gchp:
             # ---------------------------------------
             # Get the file path template for GCHP
@@ -1641,13 +1773,13 @@ def get_filepaths(datadir, collections, dates, is_gchp=False):
             if "Restart" in collection:
                 file_tmpl = os.path.join(datadir,
                                          "gcchem_internal_checkpoint.restart.")
-                separator = "_"
                 extension = ".nc4"
             else:
-                file_tmpl = os.path.join(datadir,
-                                         "GCHP.{}.".format(collection))
-                separator = "_"
-                extension = "z.nc4"
+                if gchp_format_is_legacy:
+                    file_tmpl = os.path.join(datadir,
+                                             "GCHP.{}.".format(collection))
+                else:
+                    file_tmpl = os.path.join(datadir, "GEOSChem.{}.".format(collection))
         else:
             # ---------------------------------------
             # Get the file path template for GCC
@@ -1660,17 +1792,15 @@ def get_filepaths(datadir, collections, dates, is_gchp=False):
             else:
                 file_tmpl = os.path.join(datadir,
                                          "GEOSChem.{}.".format(collection))
-                separator = "_"
-                extension = "z.nc4"
 
         # --------------------------------------------
         # Create a list of files for each date/time
         # --------------------------------------------
         for d, date in enumerate(dates):
             if is_gchp and "Restart" in collection:
-                date_time = np.datetime_as_string(date, unit="s")
+                date_time = str(np.datetime_as_string(date, unit="s"))
             else:
-                date_time = np.datetime_as_string(date, unit="m")
+                date_time = str(np.datetime_as_string(date, unit="m"))
             date_time = date_time.replace("T", separator)
             date_time = date_time.replace("-", "")
             date_time = date_time.replace(":", "")
@@ -1679,7 +1809,10 @@ def get_filepaths(datadir, collections, dates, is_gchp=False):
     return paths
 
 
-def extract_pathnames_from_log(filename, prefix_filter=""):
+def extract_pathnames_from_log(
+        filename,
+        prefix_filter=""
+):
     """
     Returns a list of pathnames from a GEOS-Chem log file.
     This can be used to get a list of files that should be
@@ -1731,7 +1864,12 @@ def extract_pathnames_from_log(filename, prefix_filter=""):
     return data_list
 
 
-def get_gcc_filepath(outputdir, collection, day, time):
+def get_gcc_filepath(
+        outputdir,
+        collection,
+        day,
+        time
+):
     '''
     Routine for getting filepath of GEOS-Chem Classic output
 
@@ -1760,7 +1898,12 @@ def get_gcc_filepath(outputdir, collection, day, time):
     return filepath
 
 
-def get_gchp_filepath(outputdir, collection, day, time):
+def get_gchp_filepath(
+        outputdir,
+        collection,
+        day,
+        time
+):
     '''
     Routine for getting filepath of GCHP output
 
@@ -1785,7 +1928,9 @@ def get_gchp_filepath(outputdir, collection, day, time):
     return filepath
 
 
-def get_nan_mask(data):
+def get_nan_mask(
+        data
+):
     """
     Create a mask with NaN values removed from an input array
 
@@ -1805,7 +1950,9 @@ def get_nan_mask(data):
     return new_data
 
 
-def all_zero_or_nan(ds):
+def all_zero_or_nan(
+        ds
+):
     """
     Return whether ds is all zeros, or all nans
 
@@ -1814,7 +1961,80 @@ def all_zero_or_nan(ds):
             Input GEOS-Chem data
     Returns:
         all_zero, all_nan: bool, bool
-            All_zero is whether ds is all zeros, all_nan is whether ds is all NaNs
+            All_zero is whether ds is all zeros, all_nan is whether ds i s all NaNs
     """
 
     return not np.any(ds), np.isnan(ds).all()
+
+
+def dataset_mean(
+        ds,
+        dim="time",
+        skipna=True
+):
+    """
+    Convenience wrapper for taking the mean of an xarray Dataset.
+
+    Args:
+       ds : xarray Dataset
+           Input data
+
+    Keyword Args:
+       dim : str
+           Dimension over which the mean will be taken.
+           Default: "time"
+       skipna : bool
+           Flag to omit missing values from the mean.
+           Default: True
+
+    Returns:
+       ds_mean : xarray Dataset or None
+           Dataset containing mean values
+           Will return None if ds is not defined
+    """
+    if ds is None:
+        return ds
+
+    if not isinstance(ds, xr.Dataset):
+        raise ValueError("Argument ds must be None or xarray.Dataset!")
+
+    with xr.set_options(keep_attrs=True):
+        return ds.mean(dim=dim, skipna=skipna)
+
+
+def dataset_reader(
+        multi_files
+):
+    """
+    Returns a function to read an xarray Dataset.
+
+    Args:
+        multi_files : bool
+            Denotes whether we will be reading multiple files into
+            an xarray Dataset.
+            Default value: False
+
+    Returns:
+         reader : either xr.open_mfdataset or xr.open_dataset
+    """
+    if multi_files:
+        reader = xr.open_mfdataset
+    else:
+        reader = xr.open_dataset
+
+    return reader
+
+def read_config_file(config_file):
+    """
+    Reads configuration information from a YAML file.
+    """
+    # Read the configuration file in YAML format
+    try:
+        print(f"Using configuration file {config_file}")
+        config = yaml.safe_load(open(config_file))
+    except Exception as err:
+        msg = f"Error reading configuration in {config_file}: {err}"
+        raise Exception(msg)
+
+    return config
+
